import os
import textwrap
import json
from glob import glob

ROOT = os.getcwd()


def main():
    with open(os.path.join(ROOT, 'dependencies.json'), 'r') as file:
        dependencies_data = json.load(file)


    tree = dependencies_data['tree']
    pypi_pins = dependencies_data['pins']['pypi']
    npm_pins = dependencies_data['pins']['npm']

    internal_packages = [
        os.path.basename(filepath)
        for filepath in glob(os.path.join(ROOT, 'packages', '*'))
    ]

    assert set(internal_packages) == set(tree.keys())
    assert set(internal_packages) == set(pypi_pins['internal'].keys())
    assert set(internal_packages) == set(npm_pins['internal'].keys())


    for package, dependency_store in tree.items():
        install_requires = []

        keys_to_keep = {'internal', 'external'}
        for where, dependencies in dependency_store.items():
            if where in keys_to_keep:
                for dependency in dependencies:
                    try:
                        pin = " " + pypi_pins[where][dependency]
                    except KeyError:
                        pin = ""

                    requirement_string = dependency + pin
                    install_requires.append(requirement_string)

        install_requires.sort()

        install_requires_filepath = os.path.join(
            ROOT, "packages", package, "src", package, "_install_requires.py")

        install_requires_contents = textwrap.dedent("""\
            install_requires = {}
        """).format(json.dumps(install_requires, indent=4))

        with open(install_requires_filepath, 'w') as file:
            file.write(install_requires_contents)


    for package, dependency_store in tree.items():
        internal_dependencies = {
            dependency: npm_pins['internal'][dependency]
            for dependency in dependency_store['internal']
        }

        package_json_filepath = os.path.join(
            ROOT, "packages", package, "package.json")

        with open(package_json_filepath, 'r') as file:
            data = json.load(file)

        try:
            external_dependencies = {
                package: pin
                for package, pin in data['dependencies'].items()
                if package not in internal_packages
            }
        except KeyError:
            external_dependencies = {}

        data['dependencies'] = {
            **internal_dependencies,
            **external_dependencies
        }

        with open(package_json_filepath, 'w') as file:
            json.dump(data, file, indent=2, sort_keys=True)



if __name__ == "__main__":
    main()


import os
import ast

from stdlib_list import stdlib_list
STDLIB = set(stdlib_list())


IMPORT_TYPES = {
    type(ast.parse('import george').body[0]),  # type: ignore
    type(ast.parse('import george as macdonald').body[0])}  # type: ignore

IMPORT_FROM_TYPES = {
    type(ast.parse('from george import macdonald').body[0])  # type: ignore
}

ALL_IMPORT_TYPES = IMPORT_TYPES.union(IMPORT_FROM_TYPES)

CONVERSIONS = {
    'attr': 'attrs',
    'PIL': 'Pillow',
    'Image': 'Pillow',
    'mpl_toolkits': 'matplotlib',
    'dateutil': 'python_dateutil'
}


def get_imports(filepath, relative_filepath, internal_packages, depth):
    with open(filepath, 'r') as file:
        data = file.read()

    parsed = ast.parse(data)
    imports = [
        node for node in ast.walk(parsed) if type(node) in ALL_IMPORT_TYPES]

    stdlib_imports = set()
    external_imports = set()
    internal_package_imports = set()
    internal_module_imports = set()
    internal_file_imports = set()

    def get_base_converted_module(name):
        name = name.split('.')[0]

        try:
            name = CONVERSIONS[name]
        except KeyError:
            pass

        return name

    def add_level_0(name):
        base_converted = get_base_converted_module(name)

        if base_converted in STDLIB:
            stdlib_imports.add(base_converted)
        elif base_converted in internal_packages:
            internal_package_imports.add(name)
        else:
            external_imports.add(base_converted)

    for an_import in imports:

        if type(an_import) in IMPORT_TYPES:
            for alias in an_import.names:
                add_level_0(alias.name)

        elif type(an_import) in IMPORT_FROM_TYPES:
            if an_import.level == 0:
                add_level_0(an_import.module)
            elif an_import.level == 1 and depth == 2:
                module_path = (
                    relative_filepath.split(os.sep)[0:2] + [an_import.module])
                internal_file_imports.add('.'.join(module_path))
            elif (
                    (an_import.level == 1 and depth == 1) or
                    (an_import.level == 2 and depth == 2)):
                module_path = (
                    relative_filepath.split(os.sep)[0:1] + [an_import.module])
                internal_module_imports.add('.'.join(module_path))
            else:
                raise ValueError(
                    "Unexpected depth and import level of relative "
                    "import")

        else:
            raise TypeError("Unexpected import type")

    return {
        'stdlib': stdlib_imports,
        'external': external_imports,
        'internal_package': internal_package_imports,
        'internal_module': internal_module_imports,
        'internal_file': internal_file_imports
    }



import os
import shutil
import subprocess

import networkx as nx


def save_dot_file(dot_contents, outfilepath):
    tred = shutil.which("tred")
    dot = shutil.which("dot")

    if not tred or not dot:
        print(
            "Graph not drawn, please install graphviz and add it to "
            "your path.\nOn Windows this is done with "
            "`choco install graphviz.portable`.\n")

        return

    with open("temp.dot", 'w') as file:
        file.write(dot_contents)

    try:
        tred_process = subprocess.Popen(
            [tred, 'temp.dot'], stdout=subprocess.PIPE)
        data = tred_process.stdout.read()
        tred_process.wait()
        with open("temp_reduced.dot", 'wb') as file:
            file.write(data)

        output = subprocess.check_output(
            [dot, '-Tsvg', 'temp_reduced.dot', '-o', 'temp.svg'])

        shutil.move("temp.svg", outfilepath)
        shutil.move("temp_reduced.dot", os.path.splitext(
            outfilepath)[0] + ".dot")
    finally:
        os.remove("temp.dot")


def remove_prefix(text, prefix):
    if text.startswith(prefix):
        return text[len(prefix):]
    else:
        return text


def get_levels(dependency_map):
    dag = dag_from_hashmap_of_lists(dependency_map)

    topological = list(nx.topological_sort(dag))

    level_map = {}
    for package in topological[::-1]:
        dependencies = nx.descendants(dag, package)
        levels = {0}
        for dependency in sorted(list(dependencies)):
            try:
                levels.add(level_map[dependency])
            except KeyError:
                pass
        max_level = max(levels)
        level_map[package] = max_level + 1

    levels = {
        level: []
        for level in range(max(level_map.values()) + 1)
    }
    for package, level in level_map.items():
        levels[level].append(package)

    return levels


def dag_from_hashmap_of_lists(dictionary):
    keys = list(dictionary.keys())
    keys.sort(reverse=True)

    dag = nx.DiGraph()

    for key in keys:
        values = sorted(dictionary[key], reverse=True)
        dag.add_node(key)
        dag.add_nodes_from(values)
        edge_tuples = [
            (key, value) for value in values
        ]
        dag.add_edges_from(edge_tuples)

    return dag


def remove_postfix(text, postfix):
    if text.endswith(postfix):
        return text[:-len(postfix)]
    else:
        return text


def convert_path_to_package(path):
    return remove_postfix(path.replace(os.sep, '.'), '.py')



import os

import networkx as nx
from copy import copy

from ..tree import PackageTree
from .utilities import save_dot_file, remove_prefix, get_levels



ROOT = os.getcwd()


def draw_directory_modules(save_directory):
    package_tree = PackageTree(os.path.join(ROOT, 'packages'))

    internal_packages = copy(package_tree.roots)
    internal_packages.remove('pymedphys')

    module_paths = [
        item
        for package in internal_packages
        for item in package_tree.digraph.neighbors(package)
    ]

    modules = {
        item: os.path.splitext(item)[0].replace(os.sep, '.')
        for item in module_paths
    }

    dependencies = {
        module.replace(os.sep, '.'): {
            '.'.join(item.split('.')[0:2])
            for item in
            package_tree.descendants_dependencies(module)['internal_module'] +
            package_tree.descendants_dependencies(module)['internal_package']
        }
        for module in modules.keys()
    }

    dependents = {  # type: ignore
        key: set() for key in dependencies.keys()
    }
    for key, values in dependencies.items():
        for item in values:
            dependents[item].add(key)  # type: ignore

    for package in internal_packages:
        build_graph_for_a_module(
            package, package_tree, dependencies, dependents, save_directory)


def build_graph_for_a_module(graphed_package, package_tree, dependencies,
                             dependents, save_directory):
    print(graphed_package)

    current_modules = sorted([
        item.replace(os.sep, '.')
        for item in package_tree.digraph.neighbors(graphed_package)
    ])

    outfilepath = os.path.join(
        save_directory, "{}.svg".format(graphed_package.replace(os.sep, '.')))

    if not current_modules:
        dot_file_contents = """
            strict digraph  {{
                subgraph cluster_0 {{
                    "";
                    label = "{}";
                    style = dashed;
                }}
            }}
        """.format(graphed_package)

        save_dot_file(dot_file_contents, outfilepath)
        return

    module_internal_relationships = {
        module.replace(os.sep, '.'): [
            '.'.join(item.split('.')[0:2])
            for item in
            package_tree.descendants_dependencies(module)['internal_module']
        ]
        for module in sorted(list(package_tree.digraph.neighbors(graphed_package)))
    }

    levels = get_levels(module_internal_relationships)


    internal_nodes = sorted(list(set(module_internal_relationships.keys())))
    external_nodes = set()
    for module in current_modules:
        external_nodes |= dependencies[module]
        external_nodes |= dependents[module]

    external_nodes = sorted(list(external_nodes))

    all_nodes = internal_nodes + external_nodes

    def simplify(text):
        text = remove_prefix(text, "{}.".format(graphed_package))
        text = remove_prefix(text, 'pymedphys_')

        return text

    label_map = {
        node: simplify(node)
        for node in all_nodes
    }


    nodes = ""

    for level in range(max(levels.keys()) + 1):
        if levels[level]:
            grouped_packages = '"; "'.join(sorted(list(levels[level])))
            nodes += """
            {{ rank = same; "{}"; }}
            """.format(grouped_packages)


    edges = ""
    current_packages = ""

    current_dependents = set()
    current_dependencies = set()


    for module in current_modules:
        current_packages += '"{}";\n'.format(module)

        for dependency in sorted(list(dependencies[module])):
            edges += '"{}" -> "{}";\n'.format(module, dependency)
            if not dependency in current_modules:
                current_dependencies.add(dependency)

        for dependent in sorted(list(dependents[module])):
            edges += '"{}" -> "{}";\n'.format(dependent, module)
            if not dependent in current_modules:
                current_dependents.add(dependent)


    external_ranks = ""
    if current_dependents:
        grouped_dependents = '"; "'.join(sorted(list(current_dependents)))
        external_ranks += '{{ rank = same; "{}"; }}\n'.format(grouped_dependents)

    if current_dependencies:
        grouped_dependencies = '"; "'.join(sorted(list(current_dependencies)))
        external_ranks += '{{ rank = same; "{}"; }}\n'.format(grouped_dependencies)

    external_labels = ""
    for node, label in label_map.items():
        external_labels += '"{}" [label="{}"];\n'.format(node, label)


    dot_file_contents = """
        strict digraph  {{
            rankdir = LR;
            subgraph cluster_0 {{
                {}
                label = "{}";
                style = dashed;
                {}
            }}
            {}
            {}
            {}
        }}
    """.format(
        current_packages, graphed_package,
        nodes, external_labels, external_ranks, edges)

    save_dot_file(dot_file_contents, outfilepath)



import os
import textwrap

import networkx as nx
from copy import copy

from ..tree import PackageTree
from .utilities import (
    save_dot_file, remove_prefix, get_levels, remove_prefix, remove_postfix,
    convert_path_to_package)



ROOT = os.getcwd()



def draw_file_modules(save_directory):
    package_tree = PackageTree(os.path.join(ROOT, 'packages'))

    internal_packages = copy(package_tree.roots)
    internal_packages.remove('pymedphys')

    directory_module_paths = [
        module_path
        for package in internal_packages
        for module_path in package_tree.digraph.neighbors(package)
    ]

    file_module_paths = [
        item
        for directory_module_path in directory_module_paths
        for item in package_tree.digraph.neighbors(directory_module_path)
    ]

    module_map = {
        item: convert_path_to_package(item)
        for item in directory_module_paths + file_module_paths
    }

    dependencies = {
        convert_path_to_package(module): {
            key: [
                convert_path_to_package(item)
                for item in descendants_dependencies
            ]
            for key, descendants_dependencies in package_tree.imports[module].items()
        }
        for module in module_map.keys()
    }

    dependents = {
        key: [] for key in dependencies.keys()
    }
    for file_module, dependency_map in dependencies.items():
        for where, values in dependency_map.items():
            for item in values:
                try:
                    dependents[item].append(file_module)
                except KeyError:
                    pass

    for directory_module_path in directory_module_paths:
        directory_module = convert_path_to_package(directory_module_path)
        print(directory_module)

        package_name = directory_module.split('.')[0]

        current_modules = [
            convert_path_to_package(item)
            for item in package_tree.digraph.neighbors(directory_module_path)
        ] + [directory_module]

        outfilepath = os.path.join(
            save_directory, "{}.svg".format(directory_module))

        if len(current_modules) <= 1:
            dot_file_contents = """
                strict digraph  {{
                    subgraph cluster_0 {{
                        "";
                        label = "{}";
                        style = dashed;
                    }}
                }}
            """.format(directory_module)

            save_dot_file(dot_file_contents, outfilepath)
            continue

        all_current_dependencies = {
            module: dependencies[module]
            for module in current_modules
        }

        keys_to_keep = {'internal_package', 'internal_module', 'internal_file'}

        current_dependencies = {
            module: [
                item
                for key, values in dependencies[module].items()
                if key in keys_to_keep
                for item in values
            ]
            for module in current_modules
        }

        current_dependents = {
            module: dependents[module]
            for module in current_modules
        }

        all_nodes = sorted(list(set([
            *current_dependencies.keys(),
            *[
                item
                for a_list in current_dependencies.values()
                for item in a_list],
            *current_dependents.keys(),
            *[
                item
                for a_list in current_dependents.values()
                for item in a_list]
        ])))

        internal_dependencies = {
            key: [
                value
                for value in values
                if value in current_modules
            ]
            for key, values in current_dependencies.items()
            if key in current_modules
        }

        internal_ranks = ""

        levels = get_levels(internal_dependencies)

        for level in range(max(levels.keys()) + 1):
            if levels[level]:
                grouped_packages = '"; "'.join(sorted(list(levels[level])))
                internal_ranks += textwrap.dedent("""\
                    {{ rank = same; "{}"; }}
                """.format(grouped_packages))

        in_same_module_other_dir = [
            node
            for node in all_nodes
            if node.startswith(package_name)
            and not node.startswith(directory_module)]

        if in_same_module_other_dir:
            in_same_module_other_dir_string = '"{}";'.format(
                '";\n"'.join(in_same_module_other_dir))
        else:
            in_same_module_other_dir_string = ''

        def simplify(text):
            text = remove_prefix(text, "{}.".format(package_name))
            text = remove_prefix(text, 'pymedphys_')

            return text

        label_map = {
            node: simplify(node)
            for node in all_nodes
        }

        label_map_str = ""
        for node, label in label_map.items():
            label_map_str += '"{}" [label="{}"];\n'.format(node, label)

        edges = ""

        for module in sorted(current_modules):
            for dependency in sorted(list(current_dependencies[module])):
                edges += '"{}" -> "{}";\n'.format(module, dependency)

            for dependent in sorted(list(current_dependents[module])):
                edges += '"{}" -> "{}";\n'.format(dependent, module)

        dot_file_contents = textwrap.dedent("""\
        strict digraph  {{
            rankdir = LR;

            subgraph cluster_0 {{
                {}
                label = "{}";
                style = dashed;

                subgraph cluster_1 {{
        {}
                    label = "{}"
                }}
            }}

        {}
        {}}}
        """).format(
            in_same_module_other_dir_string,
            package_name,
            textwrap.indent(internal_ranks, ' '*12),
            directory_module,
            textwrap.indent(label_map_str, ' '*4),
            textwrap.indent(edges, ' '*4))


        save_dot_file(dot_file_contents, outfilepath)


from .packages import draw_packages
from .directories import draw_directory_modules
from .files import draw_file_modules

def all(save_directory):
    draw_packages(save_directory)
    draw_directory_modules(save_directory)
    draw_file_modules(save_directory)


import os
import sys
import json
import subprocess
import shutil

from .build import PackageTree


def serialise_imports(imports):
    new_imports = {}
    for module_path_raw, values in imports.items():
        module_path = module_path_raw.replace(os.sep, '/')
        new_imports[module_path] = {}

        for where, a_set in values.items():
            new_imports[module_path][where] = sorted(list(a_set))

    return json.dumps(new_imports, sort_keys=True, indent=2)


def is_imports_json_up_to_date(directory):
    packages = os.path.join(directory, 'packages')
    imports_json = os.path.join(directory, 'imports.json')

    with open(imports_json) as file:
        data = json.load(file)

    file_data = json.dumps(data, sort_keys=True, indent=2)
    calced_data = serialise_imports(PackageTree(packages).imports)

    return file_data == calced_data


def commit_hook(directory):
    if not is_imports_json_up_to_date(directory):

        print(
            "\n    \033[1;31;1mThe dependency tree is out of date."
            "\033[1;32;1m Will now run `yarn tree` to update.\n"
            "    \033[1;34;1mYou will need to rerun `git commit` after "
            "this is complete.\033[0;0m\n"
        )

        sys.stdout.flush()

        yarn = shutil.which("yarn")
        git = shutil.which("git")

        subprocess.call([yarn, "tree"])

        subprocess.call([git, "add", "imports.json"])
        subprocess.call([git, "add", "dependencies.json"])
        subprocess.call([git, "add", "*package.json"])
        subprocess.call([git, "add", "*_install_requires.py"])
        subprocess.call([git, "add", "*.dot"])

        subprocess.call([git, "status"])

        print(
            "\n    \033[1;31;1mThe dependency tree was out of date.\n"
            "    \033[1;32;1mThe command `yarn tree` has been run for "
            "you.\n"
            "    \033[1;34;1mPlease rerun your commit.\033[0;0m\n"
            "    To prevent this message in the future run `yarn tree` "
            "whenever you change the dependency structure of "
            "PyMedPhys.\n")

        sys.exit(1)


def update_imports_json(directory):
    packages = os.path.join(directory, 'packages')
    imports_json = os.path.join(directory, 'imports.json')

    with open(imports_json, 'w') as file:
        file.write(serialise_imports(PackageTree(packages).imports))




import os
import json
from copy import copy, deepcopy
import difflib

import networkx as nx

from ..parse.imports import get_imports


DEPENDENCIES_JSON_FILEPATH = 'dependencies.json'
DEFAULT_EXCLUDE_DIRS = {'node_modules', '__pycache__', 'dist', '.tox', 'build'}
DEFAULT_EXCLUDE_FILES = {'__init__.py', '_version.py', '_install_requires.py'}
DEFAULT_KEYS_TO_KEEP = {'stdlib', 'internal', 'external'}


class PackageTree:
    def __init__(self, directory, exclude_dirs=None, exclude_files=None):
        if exclude_dirs is None:
            exclude_dirs = DEFAULT_EXCLUDE_DIRS

        if exclude_files is None:
            exclude_files = DEFAULT_EXCLUDE_FILES

        self.exclude_dirs = exclude_dirs
        self.exclude_files = exclude_files

        self.directory = directory

    def trim_path(self, path):
        relpath = os.path.relpath(path, self.directory)
        split = relpath.split(os.sep)
        assert split[0] == split[2]
        assert split[1] == 'src'

        if split[-1] == '__init__.py':
            split = split[:-1]

        return os.path.join(*split[2:])

    def expand_path(self, path):
        split = path.split(os.sep)
        relpath = os.path.join(split[0], 'src', path)

        if not relpath.endswith('.py'):
            relpath = os.path.join(relpath, '__init__.py')

        return os.path.join(self.directory, relpath)

    def build_directory_digraph(self):
        digraph = nx.DiGraph()
        depth = {}

        for root, dirs, files in os.walk(self._directory, topdown=True):
            dirs[:] = [d for d in dirs if d not in self.exclude_dirs]

            if '__init__.py' in files:
                module = self.trim_path(os.path.join(root, '__init__.py'))
                current_depth = module.count(os.sep) + 1
                files[:] = [f for f in files if f not in self.exclude_files]

                digraph.add_node(module)
                depth[module] = current_depth
                parent_init = os.path.join(
                    os.path.dirname(root), '__init__.py')
                if os.path.exists(parent_init):
                    digraph.add_edge(self.trim_path(parent_init), module)

                for f in files:
                    if f.endswith('.py'):
                        filepath = self.trim_path(os.path.join(root, f))
                        digraph.add_node(filepath)
                        depth[filepath] = current_depth
                        digraph.add_edge(module, filepath)

        if not digraph.nodes:
            raise ValueError('Directory provided does not contain modules')

        self.digraph = digraph
        self.depth = depth
        self.calc_properties()

    def calc_properties(self):
        self.roots = [n for n, d in self.digraph.in_degree() if d == 0]
        self.imports = {
            filepath: get_imports(
                self.expand_path(filepath), filepath, self.roots,
                self.depth[filepath])
            for filepath in self.digraph.nodes()
        }
        self._cache = {}
        self._cache['descendants_dependencies'] = {}

    @property
    def directory(self):
        return self._directory

    @directory.setter
    def directory(self, value):
        self._directory = value
        self.build_directory_digraph()

    def descendants_dependencies(self, filepath):
        try:
            return self._cache['descendants_dependencies'][filepath]
        except KeyError:
            dependencies = deepcopy(self.imports[filepath])

            for descendant in nx.descendants(self.digraph, filepath):
                for key in dependencies:
                    dependencies[key] |= self.imports[descendant][key]

            for key in dependencies:
                dependencies[key] = list(dependencies[key])
                dependencies[key].sort()

            self._cache['descendants_dependencies'][filepath] = dependencies
            return dependencies

    @property
    def package_dependencies_dict(self):
        try:
            return self._cache['package_dependencies_dict']
        except KeyError:
            key_map = {
                'internal_package': 'internal',
                'external': 'external',
                'stdlib': 'stdlib'
            }

            tree = {
                package: {
                    key_map[key]: sorted(
                        list({package.split('.')[0] for package in packages}))
                    for key, packages in self.descendants_dependencies(package).items()
                    if key in key_map.keys()
                }
                for package in self.roots
            }

            self._cache['package_dependencies_dict'] = tree
            return tree

    @property
    def package_dependencies_digraph(self):
        try:
            return self._cache['package_dependencies_digraph']
        except KeyError:
            dag = nx.DiGraph()

            for key, values in self.package_dependencies_dict.items():
                dag.add_node(key)
                dag.add_nodes_from(values['internal'])
                edge_tuples = [
                    (key, value) for value in values['internal']
                ]
                dag.add_edges_from(edge_tuples)

            self._cache['package_dependencies_digraph'] = dag
            return dag

    def is_acyclic(self):
        return nx.is_directed_acyclic_graph(self.package_dependencies_digraph)


def build_tree(directory):
    with open(DEPENDENCIES_JSON_FILEPATH, 'r') as file:
        data = json.load(file)

    data['tree'] = PackageTree(directory).package_dependencies_dict

    with open(DEPENDENCIES_JSON_FILEPATH, 'w') as file:
        json.dump(data, file, indent=2, sort_keys=True)


def test_tree(directory):
    package_tree = PackageTree(directory)
    assert package_tree.is_acyclic()
    assert_tree_unchanged(package_tree.package_dependencies_dict)


def assert_tree_unchanged(tree):
    with open(DEPENDENCIES_JSON_FILEPATH, 'r') as file:
        data = json.load(file)

    file_data = json.dumps(data['tree'], sort_keys=True, indent=2)
    calced_data = json.dumps(tree, sort_keys=True, indent=2)
    if file_data != calced_data:
        diff = difflib.unified_diff(
            file_data.split('\n'), calced_data.split('\n'))
        print('\n'.join(diff))
        raise AssertionError



import os
import json
from glob import glob
import textwrap


import semver

ROOT = os.path.dirname(os.path.dirname(os.path.abspath(os.getcwd())))

def main():
    version_filepath = glob(os.path.join(
        "src", "pymedphys*", "_version.py"))[0]
    package_name = os.path.split(os.path.dirname(version_filepath))[-1]

    with open('package.json', 'r') as file:
        data = json.load(file)

    semver_string = data['version']

    loaded_version_info = semver_string.replace(
        '.', ' ').replace('-', ' ').split(' ')

    version_info = [
        int(number)
        for number in loaded_version_info[0:3]
    ] + [''.join(loaded_version_info[3::])]  # type: ignore

    __version__ = '.'.join(
        map(str, version_info[:3])) + ''.join(version_info[3:])  # type: ignore

    version_file_contents = textwrap.dedent("""\
        version_info = {}
        __version__ = "{}"
    """.format(version_info, __version__))

    with open(version_filepath, 'w') as file:
        file.write(version_file_contents)


    semver_parsed = semver.parse(semver_string)

    if semver_parsed['major'] == 0:
        upper_limit = semver.bump_minor(semver_string)
        npm_version_prepend = "~"
    else:
        upper_limit = semver.bump_major(semver_string)
        npm_version_prepend = "^"


    dependencies_filepath = os.path.join(ROOT, "dependencies.json")


    with open(dependencies_filepath, 'r') as file:
        dependencies_data = json.load(file)


    dependencies_data['pins']['pypi']['internal'][package_name] = (
        ">= {}, < {}".format(__version__, upper_limit))
    dependencies_data['pins']['npm']['internal'][package_name] = (
        "{}{}".format(npm_version_prepend, semver_string))

    with open(dependencies_filepath, 'w') as file:
        json.dump(dependencies_data, file, indent=2, sort_keys=True)


if __name__ == "__main__":
    main()



import os
import networkx as nx

from ..tree.build import PackageTree
from .utilities import save_dot_file

ROOT = os.getcwd()


def draw_packages(save_directory):
    print('pymedphys')
    tree = PackageTree('packages').package_dependencies_dict
    tree.pop('pymedphys')
    internal_packages = tuple(tree.keys())

    keys = list(tree.keys())
    keys.sort(reverse=True)

    dag = nx.DiGraph()

    for key in keys:
        values = tree[key]

        dag.add_node(key)
        dag.add_nodes_from(values['internal'])
        edge_tuples = [
            (key, value) for value in values['internal']
        ]
        dag.add_edges_from(edge_tuples)

    levels = get_levels(dag, internal_packages)
    dot_contents = build_dot_contents(dag, levels)
    save_dot_file(dot_contents, os.path.join(save_directory, 'pymedphys.svg'))


def get_levels(dag, internal_packages):
    topological = list(nx.topological_sort(dag))

    level_map = {}
    for package in topological[::-1]:
        if package not in internal_packages:
            level_map[package] = 0
        else:
            depencencies = nx.descendants(dag, package)
            levels = {0}
            for dependency in depencencies:
                if dependency in internal_packages:
                    try:
                        levels.add(level_map[dependency])
                    except KeyError:
                        pass
            max_level = max(levels)
            level_map[package] = max_level + 1

    levels = {
        level: []
        for level in range(max(level_map.values()) + 1)
    }
    for package, level in level_map.items():
        levels[level].append(package)

    return levels


def remove_prefix(text, prefix):
    if text.startswith(prefix):
        return text[len(prefix):]
    else:
        raise ValueError("Prefix not found.")


def build_dot_contents(dag, levels):
    nodes = ""

    for level in range(max(levels.keys()) + 1):
        if levels[level]:
            trimmed_nodes = [
                remove_prefix(node, 'pymedphys_') for node in levels[level]
            ]

            grouped_packages = '; '.join(trimmed_nodes)
            nodes += """
            {{ rank = same; {}; }}
            """.format(grouped_packages)

    edges = ""

    for edge in dag.edges():
        trimmed_edge = [
            remove_prefix(node, 'pymedphys_') for node in edge
        ]
        edges += "{} -> {};\n".format(*trimmed_edge)

    dot_file_contents = """
        strict digraph  {{
            rankdir = LR;
            {}\n{}
        }}
    """.format(nodes, edges)

    return dot_file_contents
